package eliom

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Eliom_content_core.HtmlSource

Building Html tree.

See the Eliom manual for more information on dom semantics vs. functional semantics for SVG tree manipulated by client/server application.

Sourcetype +'a elt
Sourcetype 'a wrap = 'a
Sourcetype 'a list_wrap = 'a list
Sourcetype +'a attrib
Sourcetype uri = Xml.uri
Sourcemodule F : sig ... end

Typed interface for building valid HTML5 tree (functional semantics). See Html_sigs.T.

Sourcemodule D : sig ... end

Typed interface for building valid HTML5 tree (DOM semantics). See Html_sigs.T.

Typed interface for building valid HTML5 tree from React signals. HTML5's trees are automatically updated whenever corresponding signals change.

Html_sigs.T.

Sourcemodule R : sig ... end
Sourcemodule Id : sig ... end

Node identifiers

Sourcemodule Custom_data : sig ... end

Type-safe custom data for HTML5. See the examples in the manual.

Conversion of Javascript DOM elements to HTML5 elts (with DOM semantics of course). One conversion function per source type (stressed by the of_ prefix).